#ifndef BACKSTAGEINTERFACEFORUI
#define BACKSTAGEINTERFACEFORUI

#include <QWidget>
using namespace std;

typedef unsigned char UINT8;
typedef unsigned short int UINT16;
typedef unsigned int UINT32;
typedef unsigned long long int UINT64;

typedef enum 
{
	DHCP,	//DHCP方式
	STATIC	//静态
}enIpMethod;

typedef enum _unlockSignal
{ 
    DISABLE = 0,    //禁用
    UnlockSigRelay,  		//继电器
	UnlockSigWiegand26,		//韦根
	UnlockSigWiegand34,		//韦根
	UnlockSigWiegand42,		//韦根
    UnlockSigRs485,			//485
	UnlockSigButt
}enUnlockSignal;

typedef enum _assist
{ 
	FireFightingLinkage, //消防联动
	NormallyOpenOutput,	 //常开输出
	Nothing				//无
}enAssist;

typedef enum _Wiegand
{ 
	Wiegand26, 	 //韦根26
	Wiegand34,	 //韦根34
    Wiegand37	 //韦根37
}enWiegand;

typedef enum _Logo
{ 
	LogoDefault, 	 //默认
	LogoNeutral,	 //中性
    LogoServerSetting//平台配置
}enLogo;

typedef enum _recogResult
{ 
	RecogResultDefault, 	 //默认
	RecogResultShowName,	 //显示人员姓名(识别成功时使用)
	RecogResultNotRegister, //未注册(识别失败时使用)
	RecogResultCustom,		 //自定义
	RecogResultButt			//出错时返回
}enRecogResult;


typedef enum _similarityTh
{ 
	SimilarityThLoose,		//宽松
	SimilarityThGeneral,	//一般
	SimilarityThDefault,	//默认
	SimilarityThStrict,		//严格
    SimilarityThVeryStrict,	//极严格
    SimilarityThCustom      //自定义
}enSimilarityTh;

typedef enum _livingBodyTh
{ 
	LivingBodyLoose,		//宽松
	LivingBodyGeneral,	//一般
	LivingBodyStrict,		//严格
}enLivingBodyTh;

typedef enum _recogDistance
{ 
	RecogDistanceNear,		//近
	RecogDistanceMid,		//适中
	RecogDistanceFar,		//远
	RecogDistanceFarther	//更远
}enRecogDistance;

typedef enum _recogAngle
{ 
	RecogAngleLittle,	//15°
	RecogAngleMid,		//30°
	RecogAngleBig,		//45°
}enRecogAngle;

typedef enum _faceQuality
{ 
	FaceQualityBad,		//较差
	FaceQualityGood,	//适中
	FaceQualityBetter,	//较好
}enFaceQuality;

typedef enum _baudRate
{
    BaudRate9600,
    BaudRate14400,
    BaudRate19200,
    BaudRate38400,
    BaudRate57600,
    BaudRate115200
}enBaudRate;

typedef struct
{
    std::string imgPath;
    std::string name;
    std::string expTime;
    std::string regTime;
    long long personId;
    std::string idNum;
    std::string jobNum;
    std::string phoneNum;
    std::string icNum;
    std::string feature;
	int personType;
    void clear()
    {
        personId = -1;
        imgPath.clear();
        name.clear();
        expTime.clear();
        regTime.clear();
        idNum.clear();
        jobNum.clear();
        phoneNum.clear();
        icNum.clear();
        feature.clear();
		personType = -1;
    }
}stQueryPersonInfo;

typedef struct
{
    char imgPath[128];
    char name[64];
    char accessTime[64];
    bool allowPass;
}stQueryAccRedInfo;

typedef struct
{
    int type;   //0:static, 1:DHCP
    char ip[16];
    char mask[16];
    char gw[16];
    char dns[16];
    bool enable;
}stNetworkPara;

typedef struct
{
    stNetworkPara cfg;
}stNetWire;

typedef struct
{
    stNetworkPara cfg;
    char encType[16];
    char ssid[64];
    char pwd[64];
    bool isConnected;
	bool enableAp;
	char apPwd[32];
	bool isAPCfg = false;
}stNetWifi;

typedef struct
{
    char ssid[32];
    int sigLev;
    bool isEncrypt;
    bool isConnected;
}stAccPoint;

typedef struct
{
    //char provider[32];
    int provider;
    bool enable;
    bool available;
}stNet4G;

typedef struct
{
    int type;
    int interval;
    int picQty;
    char pic[10][128];
}stScreensaverCfg;

enum BlueToothDevStat_e
{
    BlueToothDevStat_idle,
    BlueToothDevStat_pairing,
    BlueToothDevStat_paired,
    BlueToothDevStat_connecting,
    BlueToothDevStat_connected
    //...
};

struct BluetoothDev_t
{
    string id;//唯一的标识?
    string name;
    //other attribute
    //e.g 当前dev的状态：已连接/连接中/空闲...
    BlueToothDevStat_e devStat;
    BluetoothDev_t(){}
    BluetoothDev_t(string _id, string _name, BlueToothDevStat_e _devStat):id(_id), name(_name), devStat(_devStat){}
    void clear(){id.clear(); name.clear();devStat = BlueToothDevStat_idle;}
};

struct BluetoothStat_t
{
    string devName;
    bool isEnable;
    list<BluetoothDev_t> pairedDev;
    list<BluetoothDev_t> availableDev;
    void clear(){devName.clear(); isEnable=false; pairedDev.clear(); availableDev.clear();}
};

struct BtDoorLock
{
    bool isEnable = false;
	int qty = 0;
	std::vector<int> manufacturer;
	std::vector<std::string> position;
	std::vector<std::string> mac;
};

typedef struct{
	std::string name;
	std::string code;
	std::string devTag;
	int type;
	int status;
}stCallDev;

typedef enum {
	audioInputPwd = 0
}enAudioIndex;

class BackstageInterfaceForUi
{
public:
    virtual ~BackstageInterfaceForUi(){}
    virtual int registerPerson(char *name, char *term, int nDataOrigin = 1) = 0;//注册人员，返回值：成功0，失败-1
    virtual int startIdentyState(bool setfaceReco = true, bool setPeInput = true) = 0;//开始识别
    virtual int stopIdentyState(bool setfaceReco = true, bool setPeInput = true) = 0;//停止识别
    
    //工程设置
    virtual int verifyFactorySettingPwd(const char *pwd) = 0;//验证密码，返回值：0、验证成功，  -1失败
    virtual int recoveryFactorySetting() = 0;//恢复出厂设置，返回值：0、成功，  -1失败
    //设备设置
    virtual int getDeviceId(char *devId, int len) = 0;//获取设备id
    virtual int getHardwareConfig(char *config, int len) = 0;//获取硬件配置
    virtual int getStorSpace(int &total, float &free) = 0;//获取存储空间
    virtual int getAppVersion(char *appVer, int len) = 0;//获取应用版本号
    virtual int getRegisterNum() = 0;//获取注册人员数
	virtual int getAccessRecordNotUpload() = 0;//获取未上传通行记录
    virtual int getAccessRecordQty() = 0;//获取通行记录条数
	virtual int getVolume() = 0;//获取音量大小
	virtual int setVolume(int vol) = 0;//设置音量大小
	virtual int getLcdBrightness() = 0;//获取屏幕亮度
	virtual int setLcdBrightness(int brightness) = 0;//设置屏幕亮度
	virtual bool getFillLightSw() = 0;//获取智能补光开关
	virtual int setFillLightSw(bool isOpen) = 0;//获取智能补光开关
	virtual int getStandbyTime() = 0;//获取待机时间
	virtual int setStandbyTime(int time) = 0;//设置待机时间
	virtual int getCloseScreenTime() = 0;//获取关屏等待时间
	virtual int setCloseScreenTime(int time) = 0;//获取关屏等待时间
	virtual int getLanguage() = 0;//获取语言
	virtual int setLanguage(int lang) = 0;//设置语言
	//网络设置
    virtual int getIP(char *ip, int len, int& type) = 0;//获取IP
	virtual int setIP(char *ip) = 0;//设置IP
	virtual enIpMethod getIpMethod() = 0;//获取IP方法
	virtual int setIpMethod(enIpMethod ipMethod) = 0;//设置IP方法
	virtual int getSubnetMask(char *mask, int len) = 0;//获取子网掩码
	virtual int setSubnetMask(char *mask) = 0;//设置子网掩码
	virtual int getGateway(char *gateway, int len) = 0;//获取默认网关
	virtual int setGateway(char *gateway) = 0;//设置默认网关
	virtual int getDNS(char *dns, int len) = 0;//获取DNS
	virtual int setDNS(char *dns) = 0;//设置DNS
	virtual int getEthMac(char *mac, int len) = 0;//获取mac
	virtual int getWifiMac(char *mac, int len) = 0;//获取wifi mac
	virtual int getServerAddr(char *addr, int len) = 0;//获取服务器ip
	virtual int setServerAddr(char *addr, bool byQrcode = false) = 0;//设置服务器ip
	virtual int testServerIP(char *addr, float &time) = 0;//测试服务器ip,成功返回0，失败-1
	virtual int getIotServerIP(char *ip, int len) = 0;//获取服务器ip
	virtual int setIotServerIP(char *ip) = 0;//设置服务器ip
	virtual int getIotPort() = 0;//获取端口号, 成功返回端口号，失败返回-1
	virtual int setIotPort(int port) = 0;//设置端口号
	virtual bool getIpShowSw() = 0;//获取IP是否在屏幕上显示的开关
	virtual int setIpShowSw(bool sw) = 0;//设置IP是否在屏幕上显示的开关
    //蓝牙设置
    //if success, return 0, else return err num;
    virtual int getBtSwitch(bool &isOpen) = 0;
    virtual int getBtName(string &devName) = 0;
    virtual int getBtPairedDev(list<BluetoothDev_t> &devList) = 0;
    virtual int getBtAvailDev(list<BluetoothDev_t> &devList) = 0;
    virtual int getBtAllInfo(BluetoothStat_t &bluetoothStat) = 0;

    virtual int btScan(list<BluetoothDev_t> &devList) = 0;//这个接口不需要吧？现在是定时的去搜索？所以点击“可用设备"条目时，直接调用getBtAvailDev就可以了？
    virtual int btSetSwitch(bool isOpen) = 0;
    virtual int btSetName(const string &devName) = 0;
    virtual int btPaire(const string &devId) = 0;
    virtual int btDisPaire(const string &devId) = 0;
    virtual int btConnect(const string &devId) = 0;
    virtual int btDisConnect(const string &devId) = 0;
    virtual int btRenameDev(const string &devId, const string &devName) = 0;

    virtual bool getParamBluetoothEnable() = 0;//获取蓝牙的开关
    virtual int setParamBluetoothEnable(bool sw) = 0;//设置蓝牙的开关
    virtual std::string getParamBluetoothName() = 0;//获取蓝牙设备名称
    virtual int setParamBluetoothName(std::string name) = 0;//设置蓝牙设备名称
	//门禁设置
	virtual int getUnlockTime() = 0;//获取开锁时间,取值范围是1-10秒。默认值为1秒
	virtual int setUnlockTime(int time) = 0;//设置开锁时间
	virtual int getUnlockDelay() = 0;//获取开锁延时,取值范围是0-10秒。默认值为0秒
	virtual int setUnlockDelay(int time) = 0;//设置开锁延时
	virtual enUnlockSignal getUnlockSig() = 0;//获取开锁信号
	virtual int setUnlockSig(enUnlockSignal value) = 0;//设置开锁信号,返回-1:不支持，-2：表示和报警信号冲突，0：成功
	virtual enAssist getAssistIn() = 0;//获取辅助输入
	virtual int setAssistIn(enAssist assist) = 0;//设置辅助输入	
	virtual enAssist getAssistOut() = 0;//获取辅助输出
	virtual int setAssistOut(enAssist assist) = 0;//设置辅助输出
	virtual enWiegand getWiegandIn() = 0;//获取韦根输入
	virtual int setWiegandIn(enWiegand wiegand) = 0;//设置韦根输入
	virtual enWiegand getWiegandOut() = 0;//获取韦根输出
	virtual int setWiegandOut(enWiegand wiegand) = 0;//设置韦根输出
    virtual enBaudRate getBaudRate() = 0;//获取波特率
    virtual int setBaudRate(enBaudRate baudRate) = 0;//设置波特率
	virtual int getWarningSig() = 0;//获取报警信号
	virtual int setWarningSig(int signal) = 0;//设置报警信号, signal,0:禁用，1：继电器。返回-1:和开门信号设置的继电器有冲突，返回0:成功
	virtual int setReverseAccessSw(bool enabled) = 0;
    virtual bool getReverseAccessSw() const = 0;
	//交互提示
	virtual enLogo getLogo() = 0;//获取logo
	virtual int setLogo(enLogo logo) = 0;//设置logo
	virtual int getCustomLogo(char *logo, int len) = 0;
	virtual enLogo getLogoTime() = 0;//获取logo+time
	virtual int setLogoTime(enLogo logo) = 0;//设置logo+time
	virtual enLogo getBackground() = 0;//获取背景图
	virtual int setBackground(enLogo logo) = 0;//设置背景图
	virtual enRecogResult getSuccessResult(char *data = nullptr, int len = 0) = 0;//获取成功结果
	virtual int setSuccessResult(enRecogResult result = RecogResultDefault, char *data = nullptr) = 0;//设置成功结果
	virtual enRecogResult getFailResult(char *data = nullptr, int len = 0) = 0;//获取失败结果
	virtual int setFailResult(enRecogResult result = RecogResultDefault, char *data = nullptr) = 0;//设置失败结果
	virtual bool getPlayAudioSw() = 0;//获取语音播报开关
	virtual int setPlayAudioSw(bool isOpen) = 0;//设置语音播报开关
    //人脸识别
    virtual bool getIdentificationSw() = 0;//获取身份核验开关
    virtual int setIdentificationSw(bool sw) = 0;//设置身份核验开关
	virtual enSimilarityTh getSimilarityTh() = 0;//获取相似度阈值
	virtual int setSimilarityTh(enSimilarityTh th) = 0;//设置相似度阈值
    virtual int setCustomSimilarity(int sim) = 0;//设置自定义识别相似度
    virtual int getCustomSimilarity() = 0;//获取自定义识别相似度
	virtual bool getLivingBodySw() = 0;//获取活体开关
	virtual int setLivingBodySw(bool isOpen) = 0;//设置活体开关	
	virtual enLivingBodyTh getLivingBodyTh() = 0;//获取活体检测阈值
	virtual int setLivingBodyTh(enLivingBodyTh th) = 0;//设置活体检测阈值	
	virtual bool getLivingBodyFailSw() = 0;//获取活体失败图片留存开关
	virtual int setLivingBodyFailSw(bool isOpen) = 0;//设置失败图片留存开关	
	virtual int getVerifyInterval() = 0;//获取验证间隔时间,取值范围0-5秒。0秒表示一人只做一次识别。大于1秒时，当上次同一人识别时间超过设定间隔时，再次进行识别
	virtual int setVerifyInterval(int time) = 0;//设置验证间隔时间	
	virtual enRecogDistance getRecogDistance() = 0;//获取识别距离
	virtual int setRecogDistance(enRecogDistance distance) = 0;//设置识别距离
	virtual enRecogAngle getRecogAngle() = 0;//获取识别角度
	virtual int setRecogAngle(enRecogAngle angle) = 0;//设置识别角度
	virtual int getMaxTraceNum() = 0;//获取人脸最大跟踪数,取值范围是1-10. 默认值为1.即每次只识别画面中最大的人脸
	virtual int setMaxTraceNum(int num) = 0;//设置人脸最大跟踪数
	virtual enFaceQuality getFaceQuality() = 0;//获取人脸质量阈值
	virtual int setFaceQuality(enFaceQuality quality) = 0;//设置人脸质量阈值
    virtual bool getDynamicFeatureSw() = 0;//获取动态底库的开关
    virtual int setDynamicFeatureSw(bool sw) = 0;//设置动态底库的开关
	//系统设置
	virtual bool getNtpServerSw() = 0;//获取NTP服务开关
	virtual int setNtpServerSw(bool isOpen) = 0;//设置NTP服务开关
	virtual int getNtpServerAddr(char *addr, int len) = 0;//获取NTP服务地址
	virtual int setNtpServerAddr(char *addr) = 0;//设置NTP服务地址
	virtual int testNtpServer(char *addr, float &time) = 0;//测试NTP服务，	成功返回0，失败-1
	virtual int getNtpServerPort() = 0;//获取NTP端口，成功返回端口号，失败返回-1
	virtual int setNtpServerPort(int port) = 0;//设置NTP服务端口
    virtual int getNtpCheckInterval() = 0;///////////////////////////////////////////
    virtual int setNtpCheckInterval(int interval) = 0;///////////////////////////////////////////
    virtual int getNtpTimeZone(char *timeZone, int len) = 0;///////////////////////////////////////////
    virtual int setNtpTimeZone(char* timeZone) = 0;///////////////////////////////////////////
	virtual int setVerifyFactorySettingPwd(char *pwd) = 0;//设置工程密码，密码要求：数字0-9，字母a-z，A-Z，共6位。默认密码为666666
	virtual int testHardware() = 0;//硬件检测,包括：a)双摄像头的回显，				b)补光灯的点亮,c)继电器开关,d)刷卡信息的显示
	virtual int cleanData() = 0;//清除数据
	virtual int devReboot() = 0;//重启设备

    virtual int startCapture() = 0;//start capture on register

	virtual int deletePerson(long long personID) = 0;

	/*query person ID
	*pPersonIDBuff     : [out]person id list 
	*nBuffCount 	   : [in] buff size
	*startID		   : [in] start id, the param greater zero
	*/
	virtual int queryPersonList(long long* pPersonIDBuff, int nBuffCount, long long startID, int& nRealPerson) = 0;
    virtual int clearPeople() = 0;
    virtual unsigned queryPerson(const char* searchText) = 0;
    virtual unsigned queryPerson(const char* searchText, int pageIndex, stQueryPersonInfo personRet[], int personRetSize) = 0;
    virtual unsigned queryAccessRecord(const char* name, const char* time) = 0;
    virtual unsigned queryAccessRecord(const char* name, const char* time, int pageIndex, stQueryAccRedInfo recordRet[], int recordRetSize) = 0;
    virtual int getPersonListState(int &sum, unsigned long long &maxPersonID) = 0;
    virtual int wakeUpFromScreenSaverOrScreenOff() = 0;

    virtual int delPerson(long long personId) = 0;
    virtual int modifyPerson(stQueryPersonInfo &personInfo) = 0;
    virtual int addPerson(stQueryPersonInfo &personInfo) = 0;
    virtual int snapFace(std::string &feature, std::string &savePath) = 0;

    virtual bool getMaskCheckSwitch() = 0;
    virtual int setMaskCheckSwitch(const bool sw) = 0;

    virtual bool getNoMaskNoPassSwitch() = 0;
    virtual int setNoMaskNoPassSwitch(const bool sw) = 0;

    virtual bool getTemperatureCheckSwitch() = 0;
    virtual int setTemperatureCheckSwitch(const bool sw) = 0;

    virtual bool getNoTemperatureNoPassSwitch() = 0;
    virtual int setNoTemperatureNoPassSwitch(const bool sw) = 0;

    virtual float getLowTemperature() = 0;
    virtual int setLowTemperature(const float temp) = 0;
    virtual float getHighTemperature() = 0;
    virtual int setHighTemperature(const float temp) = 0;
	virtual bool getQRCodeSwitch() = 0;
	virtual int setQRCodeSwitch(const bool sw) = 0;

    virtual int setDateTime(const char *dateTime) = 0;

    virtual bool getDataEncryptSw() = 0;
    virtual int setDataEncryptSw(bool sw) = 0;
    virtual bool getAutoRebootSw() = 0;
    virtual int setAutoRebootSw(bool sw) = 0;
    virtual int getAutoRebootTime(char* time, int len) = 0;
    virtual int setAutoRebootTime(const char *time) = 0;
    virtual int getAutoRebootInterval() = 0;
    virtual int setAutoRebootInterval(int interval) = 0;

    virtual int ntpTimeCorrect() = 0;
    virtual int ntpTimeCorrectForDevTest(const std::string& ntpServerIp) = 0;

    virtual int getNetWireCfg(stNetWire& wireCfg) = 0;
    virtual int setNetWireCfg(const stNetWire& wireCfg) = 0;
    virtual int setNetWireEnable(bool enable) = 0;

    virtual int getNetWifiCfg(stNetWifi& wifiCfg) = 0;
    virtual int setNetWifiCfg(const stNetWifi& wifiCfg) = 0;
    virtual int setNetWifiEnable(bool enable) = 0;
    virtual int getSsidList(stAccPoint accPointList[], int accPointListSize) = 0;
    virtual int claerNetWifiCfg() = 0;

    virtual int getNet4GCfg(stNet4G& n4gCfg) = 0;
    virtual int setNet4GEnable(bool enable) = 0;
    virtual int reboot4G() = 0;

    virtual int getScreenSaverConfig(stScreensaverCfg& cfg) = 0;

	virtual int sendPassword(const char *password) = 0;
	virtual void resetDecisionCenterTimer() = 0;
	virtual int pauseDataFlow() = 0;
    virtual int resumeDataFlow() = 0;

	virtual int candidateChoosedInPopupWindow(int index) = 0;

	virtual bool isWiegandIoTestOk() = 0;
	virtual bool isRelayAndGpioTestOk() = 0;
	virtual bool isRs485TestOk() = 0;
	virtual bool isReaderUnderScreenTestOk() = 0;
	virtual bool isWifiTestOk() = 0;
	virtual bool isRtcTestOk() = 0;
	virtual bool isWireEthTestOk() = 0;
	virtual bool is4gTestOk() = 0;
	virtual bool isSerial1TestOk() = 0;

	virtual void screenTest() = 0;
	virtual void speakerTest() = 0;
	virtual void microphoneTest(bool record) = 0;
    virtual void audioSwitchTest() = 0;
    virtual void tpTest() = 0;
	virtual void irCamTest() = 0;
	virtual void irLedTest() = 0;
	virtual void ageingTest(unsigned hours) = 0;	
	virtual int getMacFromMes(const char* mesAddrAndProdNo, const char* pcbSeq, std::string& mac) = 0;
	virtual bool isAgeing(std::string& hours) = 0;	
	virtual int getKeyDataMac(std::string& mac) = 0;

	virtual int getRecoUiAdConfig(stScreensaverCfg& cfg) = 0;
	virtual int recoUiTypeSwitch(unsigned x, unsigned y, unsigned w, unsigned h) = 0;

	virtual int getDevPwd(int& type, std::string& pwd) = 0;
	virtual int setLocalDevPwd(const std::string& pwd) = 0;

	virtual int getKeyDataHardware(std::string& hardware) = 0;
	virtual int setKeyDataHardwareAndUpdataFunction(const std::string& hardware) = 0;

	virtual int setSetupAlreadyFlag() = 0;
	virtual int getParkInfo(std::string& info) = 0;
	virtual int setParkInfo(const std::string& info) = 0;
	virtual int getDevPosi(std::string& posi) = 0;
	virtual int setDevPosi(const std::string& posi) = 0;

	virtual int getRs485Cfg() = 0;
	virtual int setRs485Cfg(int cfg) = 0;
	virtual int getRs485Prot1Addr() = 0;
	virtual int setRs485Prot1Addr(int addr) = 0;
	virtual int getRs485Prot3Cfg(int& handCfg, int& feetCfg) = 0;
	virtual int setRs485Prot3Cfg(int handCfg, int feetCfg) = 0;
	virtual int getGPIOCfg() = 0;
	virtual int setGPIOCfg(int cfg) = 0;
	virtual int getIDFaceVerifySw() = 0;
	virtual int setIDFaceVerifySw(bool sw) = 0;
	virtual int getBtLockCfg(BtDoorLock& cfg) = 0;
	virtual int setBtLockCfg(const BtDoorLock& cfg) = 0;
	virtual int getBtLockRemainingBatPwr(const std::string& mac) = 0;
	virtual bool getHealthCodeSw() = 0;
	virtual int setHealthCodeSw(bool sw) = 0;
	virtual bool getHealthCodeNoFaceRequired() = 0;
	virtual int setHealthCodeNoFaceRequired(bool sw) = 0;
	virtual bool getHealthCodeAllowStranger() = 0;
	virtual int setHealthCodeAllowStranger(bool sw) = 0;
	virtual int getHealthCodePassCfg(int& codeColorCfg, int& c19tCfg, int& tripCfg, int& abtCfg) = 0;
	virtual int setHealthCodePassCfg(int codeColorCfg, int c19tCfg, int tripCfg, int abtCfg) = 0;
	virtual std::string getUnderScreenNfcModuleId() = 0;	 
	virtual std::string getExtendNfcModuleId() = 0;
	virtual bool isIdModuleInfoUploaded() = 0;
	virtual bool isDevSettingDisable() = 0;
	virtual void setAntiDemolitionAvail(bool avail, bool upload) = 0;
	virtual bool getAntiDemolitionSw() = 0;
	virtual void setAntiDemolitionSw(bool sw) = 0;
	virtual int getAntiDemolitionOutput() = 0;
	virtual void setAntiDemolitionOutput(int output) = 0;
	virtual int getGpio4Cfg() = 0;
	virtual void setGpio4Cfg(int cfg) = 0;
	virtual int getGpio5Cfg() = 0;
	virtual void setGpio5Cfg(int cfg) = 0;
	virtual int getDoorCloseOvertime() = 0;
	virtual void setDoorCloseOvertime(int overtime) = 0;
	virtual bool getDisableDevSettingSw() = 0;
	virtual void setDisableDevSettingSw(bool sw) = 0;
	virtual int getTicketCheckCountToday() = 0;
	virtual int getTicketModePeopleCountType() = 0;
    virtual std::string utfRawStrToEncodeStr(const std::string &str) = 0;
	virtual int getCallDevList() = 0;
	virtual int queryCallDevList(const std::string& text, std::vector<stCallDev>& devList) = 0;
	virtual int callDev(const std::string& devTag, int devType) = 0;
	virtual int callAbort() = 0;
	virtual int callHangUp() = 0;
	virtual int callReject() = 0;
	virtual int callAccept() = 0;
	virtual int callSetVideoStream(bool enable) = 0;
	virtual int openCallRemoteDoor() = 0;
	virtual void verifyAbort() = 0;
	virtual bool getShowIdAndVerSw() = 0;
	virtual int setShowIdAndVerSw(bool sw) = 0;
	virtual int playAudio(enAudioIndex index) = 0;
	virtual int userItemChoose(int index) = 0;
};

#endif // BACKSTAGEINTERFACEFORUI

