#ifndef RECOUIRECOGNIZESCENE_H
#define RECOUIRECOGNIZESCENE_H

#include <QWidget>
#include "UiTools.h"
#include "recoUiRecognize.h"


class recoUiRecognizeScene : public PageAcceptMouseAndTouch
{
    Q_OBJECT
public:
    static const int TOP_MSG_WIDGET_HEIGHT[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];
    static const QString TOP_MSG_WIDGET_BG[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];
	static const int SWIP_CARD_WIDGET_WIDTH[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];
	static const int SWIP_CARD_WIDGET_HEIGHT[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];
	static const QString SWIP_CARD_DAY_MODE_WIDGET_BG[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];
	static const QString SWIP_CARD_NIGHT_MODE_WIDGET_BG[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];
	static const QString PAGE_BG[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];
	static const QString PAGE_STATUS_BG[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];

    explicit recoUiRecognizeScene(bool isNightMode, bool hasPwdBtn, QWidget *parent = nullptr);
	virtual ~recoUiRecognizeScene();
	void setBackstageUiinterface(BackstageInterfaceForUi *interface);
	int setCamVideoWind();
	void updateBg();
	void uiReset();
	void setTicketCheckCount(int source, int count);

signals:
    void signalChangePage();
    void signalShowPwdPage(const QString& name);
	void signalScreenTouch();

public slots:
    void slotsNoSuccessPage(int nPage);
    void slotsSuccessPage(const int nPage, const QString& name);
    void slotsDrawScanner(const int nPage, const bool faceAvailable);
    //void handleRecoResShowTime();
    void slotsInteractionChange(const INTERACION_OPTION& interaction, int nSelectRes);
    void slotShowNormalTemp(const float temp, const bool isC);
    void slotShowAbnormalTemp(const float temp, const bool isHigh, const bool isC);
    void slotUpdateLogo(const int);
    void slotUpdatePlatformConnectionStatus(const int);
    void slotsResetRecoUi();
    //void slotShowQRCodeMsg(const int);
    //void slotChangeRecoUiMode();
    //void slotTimerQRCodeScanline();
    //void slotQRCodeSwitchToStandbyStatusFromOvertimeStatus();
    void slotUpdateRecoUiIpDisplay(const bool);
    //void slotShowFeatureUpdateProgress(const unsigned current, const unsigned all);
    //void slotCloseFeatureUpdateProgress();
    void slotShowRecoUiMask(int maskType);
    void slotNotiMsg(const int nPage, const QString& msg, int color);
    void slotLongPressShowSetting();
    void slotCenterMsg(const int nPage, const QString& msg);
    void slotPwdBtnClicked();
    void slotSetUserPwdButton(bool visible);
    //void slotShowOrHideFaceFrame(bool show);
    void slotShowFinalResult(bool pass);
	
private:
	//SceneMsgWidget* m_wgtSceneMsg;
	MsgWidget* m_wgtDevMsg;
    QWidget* m_wgtBack;	
	QWidget* m_wgtBtnPwd;
	QWidget* m_wgtCardUnderScreen = nullptr;
	//QLabel* m_wgtStatusTip;
    CenterMsgWidget* m_widget_cMsg;
	PersonWidget* m_widget_person;

	//QBrush m_pageIdle;
	//QBrush m_pageStatusTip;
	QBrush m_pageReco;
	
    QString m_name;

    int m_clickCount;
    QTime _preTime;
    bool isTouched;

    QString recoUiTextCfg(RecoUiTextCfg::enTextType type);
	bool eventFilter(QObject *watched, QEvent *event);
    bool event(QEvent *event);
    bool multiPressToSettingPage();
    bool longPressToSettingPage();
	void setMsgWidgetBg(int bgType);
	void setCardReaderUnderScreenInstructions(bool isNightMode);
	void setBg(const QBrush& brush);
	void showStatusTip(const QString& tip);
};

class recoUiSceneResult : public QWidget
{
    Q_OBJECT
public:
	static const QString PAGE_BG[UiConfig::SUPPORT_SCREEN_SIZE_QUANTITY];

    explicit recoUiSceneResult(QWidget *parent = nullptr);
	void showResult(bool pass, const QString& result, const QString& businessInfo);
	bool updatePersonNumAccessed(int personAccess, int personAll);
	void setTicketCheckCount(int source, int count);
	bool isPassing() const;
	void resetPersonCount();
	void setPersonCountType(int type);
	
signals:

public slots:

private:
	SceneMsgWidget* m_wgtSceneMsg;
	QLabel* m_labelResult;	
	
	QLabel* m_labelBussInfo;
	QLabel* m_labelBussPersonNum;

	QBrush m_bgWhiteList;
	QBrush m_bgSceneOk;
	QBrush m_bgSceneFailed;
	
	//QWidget* m_accMsg;
	//QLabel* m_labelAcc;
	//QLabel* m_labelAccRes;
	int m_personAccessed;
	int m_personAll;
};

#endif // RECOUIRECOGNIZESCENE_H
